/* Generated by Cython 0.23.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_23_3"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if !defined(CYTHON_USE_PYLONG_INTERNALS) && CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x02070000
#define CYTHON_USE_PYLONG_INTERNALS 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if PY_VERSION_HEX >= 0x030500B1
#define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
#elif CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
typedef struct {
    unaryfunc am_await;
    unaryfunc am_aiter;
    unaryfunc am_anext;
} __Pyx_PyAsyncMethodsStruct;
#define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
#else
#define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#define __Pyx_void_to_None(void_result) (void_result, Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__bzrlib___readdir_pyx
#define __PYX_HAVE_API__bzrlib___readdir_pyx
#include "python-compat.h"
#include "errno.h"
#include "unistd.h"
#include "stdlib.h"
#include "sys/types.h"
#include "sys/stat.h"
#include "fcntl.h"
#include "dirent.h"
#include "readdir.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "bzrlib/_readdir_pyx.pyx",
};

/*--- Type declarations ---*/
struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat;
struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader;

/* "bzrlib/_readdir_pyx.pyx":131
 * 
 * 
 * cdef class _Stat:             # <<<<<<<<<<<<<<
 *     """Represent a 'stat' result."""
 * 
 */
struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat {
  PyObject_HEAD
  struct stat _st;
};


/* "bzrlib/_readdir_pyx.pyx":174
 * _safe_utf8 = osutils.safe_utf8
 * 
 * cdef class UTF8DirReader:             # <<<<<<<<<<<<<<
 *     """A dir reader for utf8 file systems."""
 * 
 */
struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader {
  PyObject_HEAD
  struct __pyx_vtabstruct_6bzrlib_12_readdir_pyx_UTF8DirReader *__pyx_vtab;
};



struct __pyx_vtabstruct_6bzrlib_12_readdir_pyx_UTF8DirReader {
  PyObject *(*_kind_from_mode)(struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *, int);
};
static struct __pyx_vtabstruct_6bzrlib_12_readdir_pyx_UTF8DirReader *__pyx_vtabptr_6bzrlib_12_readdir_pyx_UTF8DirReader;

/* --- Runtime support code (head) --- */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

#include <string.h>

static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);

static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb);
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb);

static int __Pyx_SetVtable(PyObject *dict, void *vtable);

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ino_t(ino_t value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_off_t(off_t value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static int __Pyx_check_binary_version(void);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_6bzrlib_12_readdir_pyx_13UTF8DirReader__kind_from_mode(CYTHON_UNUSED struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *__pyx_v_self, int __pyx_v_mode); /* proto*/

/* Module declarations from 'bzrlib._readdir_pyx' */
static PyTypeObject *__pyx_ptype_6bzrlib_12_readdir_pyx__Stat = 0;
static PyTypeObject *__pyx_ptype_6bzrlib_12_readdir_pyx_UTF8DirReader = 0;
static PyObject *__pyx_v_6bzrlib_12_readdir_pyx__directory = 0;
static PyObject *__pyx_v_6bzrlib_12_readdir_pyx__chardev = 0;
static PyObject *__pyx_v_6bzrlib_12_readdir_pyx__block = 0;
static PyObject *__pyx_v_6bzrlib_12_readdir_pyx__file = 0;
static PyObject *__pyx_v_6bzrlib_12_readdir_pyx__fifo = 0;
static PyObject *__pyx_v_6bzrlib_12_readdir_pyx__symlink = 0;
static PyObject *__pyx_v_6bzrlib_12_readdir_pyx__socket = 0;
static PyObject *__pyx_v_6bzrlib_12_readdir_pyx__unknown = 0;
static PyObject *__pyx_v_6bzrlib_12_readdir_pyx__safe_utf8 = 0;
static PyObject *__pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(int, char *, PyObject *); /*proto*/
static PyObject *__pyx_f_6bzrlib_12_readdir_pyx__read_dir(PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "bzrlib._readdir_pyx"
int __pyx_module_is_main_bzrlib___readdir_pyx = 0;

/* Implementation of 'bzrlib._readdir_pyx' */
static PyObject *__pyx_builtin_Exception;
static PyObject *__pyx_builtin_OSError;
static char __pyx_k_[] = "";
static char __pyx_k__2[] = "/";
static char __pyx_k__5[] = ".";
static char __pyx_k_os[] = "os";
static char __pyx_k_sys[] = "sys";
static char __pyx_k_top[] = "top";
static char __pyx_k_fifo[] = "fifo";
static char __pyx_k_file[] = "file";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_open[] = "open: ";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_block[] = "block";
static char __pyx_k_chdir[] = "chdir: ";
static char __pyx_k_lstat[] = "lstat: ";
static char __pyx_k_bzrlib[] = "bzrlib";
static char __pyx_k_import[] = "__import__";
static char __pyx_k_prefix[] = "prefix";
static char __pyx_k_socket[] = "socket";
static char __pyx_k_OSError[] = "OSError";
static char __pyx_k_chardev[] = "chardev";
static char __pyx_k_opendir[] = "opendir: ";
static char __pyx_k_osutils[] = "osutils";
static char __pyx_k_readdir[] = "readdir: ";
static char __pyx_k_st_mode[] = "st_mode";
static char __pyx_k_st_size[] = "st_size";
static char __pyx_k_symlink[] = "symlink";
static char __pyx_k_unknown[] = "unknown";
static char __pyx_k_closedir[] = "closedir: ";
static char __pyx_k_st_ctime[] = "st_ctime";
static char __pyx_k_st_mtime[] = "st_mtime";
static char __pyx_k_Exception[] = "Exception";
static char __pyx_k_directory[] = "directory";
static char __pyx_k_safe_utf8[] = "safe_utf8";
static char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static char __pyx_k_failed_to_strcat[] = "failed to strcat";
static char __pyx_k_return_to_orig_dir[] = "return to orig_dir: ";
static char __pyx_k_Wrapper_for_readdir_which_return[] = "Wrapper for readdir which returns files ordered by inode.";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_n_s_Exception;
static PyObject *__pyx_n_s_OSError;
static PyObject *__pyx_kp_s__2;
static PyObject *__pyx_kp_s__5;
static PyObject *__pyx_n_s_block;
static PyObject *__pyx_n_s_bzrlib;
static PyObject *__pyx_n_s_chardev;
static PyObject *__pyx_n_s_directory;
static PyObject *__pyx_kp_s_failed_to_strcat;
static PyObject *__pyx_n_s_fifo;
static PyObject *__pyx_n_s_file;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_os;
static PyObject *__pyx_n_s_osutils;
static PyObject *__pyx_n_s_prefix;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_safe_utf8;
static PyObject *__pyx_n_s_socket;
static PyObject *__pyx_n_s_st_ctime;
static PyObject *__pyx_n_s_st_mode;
static PyObject *__pyx_n_s_st_mtime;
static PyObject *__pyx_n_s_st_size;
static PyObject *__pyx_n_s_symlink;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_top;
static PyObject *__pyx_n_s_unknown;
static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_6st_dev___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_6st_ino___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_7st_mode___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_8st_ctime___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_8st_mtime___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_7st_size___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat___repr__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_13UTF8DirReader_kind_from_mode(struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *__pyx_v_self, int __pyx_v_mode); /* proto */
static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_13UTF8DirReader_2top_prefix_to_starting_dir(CYTHON_UNUSED struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *__pyx_v_self, PyObject *__pyx_v_top, PyObject *__pyx_v_prefix); /* proto */
static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_13UTF8DirReader_4read_dir(struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *__pyx_v_self, PyObject *__pyx_v_prefix, PyObject *__pyx_v_top); /* proto */
static PyObject *__pyx_tp_new_6bzrlib_12_readdir_pyx__Stat(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6bzrlib_12_readdir_pyx_UTF8DirReader(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;

/* "bzrlib/_readdir_pyx.pyx":137
 * 
 *     property st_dev:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_dev
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_6st_dev_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_6st_dev_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_6st_dev___get__(((struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_6st_dev___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "bzrlib/_readdir_pyx.pyx":138
 *     property st_dev:
 *         def __get__(self):
 *             return self._st.st_dev             # <<<<<<<<<<<<<<
 * 
 *     property st_ino:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_st.st_dev); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 138; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":137
 * 
 *     property st_dev:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_dev
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzrlib._readdir_pyx._Stat.st_dev.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":141
 * 
 *     property st_ino:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_ino
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_6st_ino_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_6st_ino_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_6st_ino___get__(((struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_6st_ino___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "bzrlib/_readdir_pyx.pyx":142
 *     property st_ino:
 *         def __get__(self):
 *             return self._st.st_ino             # <<<<<<<<<<<<<<
 * 
 *     property st_mode:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_ino_t(__pyx_v_self->_st.st_ino); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":141
 * 
 *     property st_ino:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_ino
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzrlib._readdir_pyx._Stat.st_ino.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":145
 * 
 *     property st_mode:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_mode
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_7st_mode_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_7st_mode_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_7st_mode___get__(((struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_7st_mode___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "bzrlib/_readdir_pyx.pyx":146
 *     property st_mode:
 *         def __get__(self):
 *             return self._st.st_mode             # <<<<<<<<<<<<<<
 * 
 *     property st_ctime:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_st.st_mode); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 146; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":145
 * 
 *     property st_mode:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_mode
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzrlib._readdir_pyx._Stat.st_mode.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":149
 * 
 *     property st_ctime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_ctime
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_8st_ctime_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_8st_ctime_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_8st_ctime___get__(((struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_8st_ctime___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "bzrlib/_readdir_pyx.pyx":150
 *     property st_ctime:
 *         def __get__(self):
 *             return self._st.st_ctime             # <<<<<<<<<<<<<<
 * 
 *     property st_mtime:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_st.st_ctime); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 150; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":149
 * 
 *     property st_ctime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_ctime
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzrlib._readdir_pyx._Stat.st_ctime.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":153
 * 
 *     property st_mtime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_mtime
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_8st_mtime_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_8st_mtime_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_8st_mtime___get__(((struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_8st_mtime___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "bzrlib/_readdir_pyx.pyx":154
 *     property st_mtime:
 *         def __get__(self):
 *             return self._st.st_mtime             # <<<<<<<<<<<<<<
 * 
 *     property st_size:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->_st.st_mtime); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":153
 * 
 *     property st_mtime:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_mtime
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzrlib._readdir_pyx._Stat.st_mtime.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":157
 * 
 *     property st_size:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_size
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_7st_size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_7st_size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_7st_size___get__(((struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat_7st_size___get__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "bzrlib/_readdir_pyx.pyx":158
 *     property st_size:
 *         def __get__(self):
 *             return self._st.st_size             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_off_t(__pyx_v_self->_st.st_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":157
 * 
 *     property st_size:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._st.st_size
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzrlib._readdir_pyx._Stat.st_size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":160
 *             return self._st.st_size
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         """Repr is the same as a Stat object.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_1__repr__(PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_6bzrlib_12_readdir_pyx_5_Stat___repr__[] = "Repr is the same as a Stat object.\n\n        (mode, ino, dev, nlink, uid, gid, size, None(atime), mtime, ctime)\n        ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_6bzrlib_12_readdir_pyx_5_Stat___repr__;
#endif
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_1__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_6bzrlib_12_readdir_pyx_5_Stat___repr__(((struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_5_Stat___repr__(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "bzrlib/_readdir_pyx.pyx":165
 *         (mode, ino, dev, nlink, uid, gid, size, None(atime), mtime, ctime)
 *         """
 *         return repr((self.st_mode, 0, 0, 0, 0, 0, self.st_size, None,             # <<<<<<<<<<<<<<
 *                      self.st_mtime, self.st_ctime))
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_st_mode); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_st_size); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);

  /* "bzrlib/_readdir_pyx.pyx":166
 *         """
 *         return repr((self.st_mode, 0, 0, 0, 0, 0, self.st_size, None,
 *                      self.st_mtime, self.st_ctime))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_st_mtime); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_st_ctime); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);

  /* "bzrlib/_readdir_pyx.pyx":165
 *         (mode, ino, dev, nlink, uid, gid, size, None(atime), mtime, ctime)
 *         """
 *         return repr((self.st_mode, 0, 0, 0, 0, 0, self.st_size, None,             # <<<<<<<<<<<<<<
 *                      self.st_mtime, self.st_ctime))
 * 
 */
  __pyx_t_5 = PyTuple_New(10); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_0);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_int_0);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_int_0);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_int_0);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_5, 5, __pyx_int_0);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 6, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_5, 7, Py_None);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 8, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 9, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyObject_Repr(__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":160
 *             return self._st.st_size
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         """Repr is the same as a Stat object.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("bzrlib._readdir_pyx._Stat.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":177
 *     """A dir reader for utf8 file systems."""
 * 
 *     def kind_from_mode(self, int mode):             # <<<<<<<<<<<<<<
 *         """Get the kind of a path from a mode status."""
 *         return self._kind_from_mode(mode)
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_13UTF8DirReader_1kind_from_mode(PyObject *__pyx_v_self, PyObject *__pyx_arg_mode); /*proto*/
static char __pyx_doc_6bzrlib_12_readdir_pyx_13UTF8DirReader_kind_from_mode[] = "Get the kind of a path from a mode status.";
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_13UTF8DirReader_1kind_from_mode(PyObject *__pyx_v_self, PyObject *__pyx_arg_mode) {
  int __pyx_v_mode;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("kind_from_mode (wrapper)", 0);
  assert(__pyx_arg_mode); {
    __pyx_v_mode = __Pyx_PyInt_As_int(__pyx_arg_mode); if (unlikely((__pyx_v_mode == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 177; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzrlib._readdir_pyx.UTF8DirReader.kind_from_mode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzrlib_12_readdir_pyx_13UTF8DirReader_kind_from_mode(((struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *)__pyx_v_self), ((int)__pyx_v_mode));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_13UTF8DirReader_kind_from_mode(struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *__pyx_v_self, int __pyx_v_mode) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("kind_from_mode", 0);

  /* "bzrlib/_readdir_pyx.pyx":179
 *     def kind_from_mode(self, int mode):
 *         """Get the kind of a path from a mode status."""
 *         return self._kind_from_mode(mode)             # <<<<<<<<<<<<<<
 * 
 *     cdef _kind_from_mode(self, int mode):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_6bzrlib_12_readdir_pyx_UTF8DirReader *)__pyx_v_self->__pyx_vtab)->_kind_from_mode(__pyx_v_self, __pyx_v_mode); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":177
 *     """A dir reader for utf8 file systems."""
 * 
 *     def kind_from_mode(self, int mode):             # <<<<<<<<<<<<<<
 *         """Get the kind of a path from a mode status."""
 *         return self._kind_from_mode(mode)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("bzrlib._readdir_pyx.UTF8DirReader.kind_from_mode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":181
 *         return self._kind_from_mode(mode)
 * 
 *     cdef _kind_from_mode(self, int mode):             # <<<<<<<<<<<<<<
 *         # Files and directories are the most common - check them first.
 *         if S_ISREG(mode):
 */

static PyObject *__pyx_f_6bzrlib_12_readdir_pyx_13UTF8DirReader__kind_from_mode(CYTHON_UNUSED struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *__pyx_v_self, int __pyx_v_mode) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_kind_from_mode", 0);

  /* "bzrlib/_readdir_pyx.pyx":183
 *     cdef _kind_from_mode(self, int mode):
 *         # Files and directories are the most common - check them first.
 *         if S_ISREG(mode):             # <<<<<<<<<<<<<<
 *             return _file
 *         if S_ISDIR(mode):
 */
  __pyx_t_1 = (S_ISREG(__pyx_v_mode) != 0);
  if (__pyx_t_1) {

    /* "bzrlib/_readdir_pyx.pyx":184
 *         # Files and directories are the most common - check them first.
 *         if S_ISREG(mode):
 *             return _file             # <<<<<<<<<<<<<<
 *         if S_ISDIR(mode):
 *             return _directory
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_6bzrlib_12_readdir_pyx__file);
    __pyx_r = __pyx_v_6bzrlib_12_readdir_pyx__file;
    goto __pyx_L0;

    /* "bzrlib/_readdir_pyx.pyx":183
 *     cdef _kind_from_mode(self, int mode):
 *         # Files and directories are the most common - check them first.
 *         if S_ISREG(mode):             # <<<<<<<<<<<<<<
 *             return _file
 *         if S_ISDIR(mode):
 */
  }

  /* "bzrlib/_readdir_pyx.pyx":185
 *         if S_ISREG(mode):
 *             return _file
 *         if S_ISDIR(mode):             # <<<<<<<<<<<<<<
 *             return _directory
 *         if S_ISCHR(mode):
 */
  __pyx_t_1 = (S_ISDIR(__pyx_v_mode) != 0);
  if (__pyx_t_1) {

    /* "bzrlib/_readdir_pyx.pyx":186
 *             return _file
 *         if S_ISDIR(mode):
 *             return _directory             # <<<<<<<<<<<<<<
 *         if S_ISCHR(mode):
 *             return _chardev
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_6bzrlib_12_readdir_pyx__directory);
    __pyx_r = __pyx_v_6bzrlib_12_readdir_pyx__directory;
    goto __pyx_L0;

    /* "bzrlib/_readdir_pyx.pyx":185
 *         if S_ISREG(mode):
 *             return _file
 *         if S_ISDIR(mode):             # <<<<<<<<<<<<<<
 *             return _directory
 *         if S_ISCHR(mode):
 */
  }

  /* "bzrlib/_readdir_pyx.pyx":187
 *         if S_ISDIR(mode):
 *             return _directory
 *         if S_ISCHR(mode):             # <<<<<<<<<<<<<<
 *             return _chardev
 *         if S_ISBLK(mode):
 */
  __pyx_t_1 = (S_ISCHR(__pyx_v_mode) != 0);
  if (__pyx_t_1) {

    /* "bzrlib/_readdir_pyx.pyx":188
 *             return _directory
 *         if S_ISCHR(mode):
 *             return _chardev             # <<<<<<<<<<<<<<
 *         if S_ISBLK(mode):
 *             return _block
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_6bzrlib_12_readdir_pyx__chardev);
    __pyx_r = __pyx_v_6bzrlib_12_readdir_pyx__chardev;
    goto __pyx_L0;

    /* "bzrlib/_readdir_pyx.pyx":187
 *         if S_ISDIR(mode):
 *             return _directory
 *         if S_ISCHR(mode):             # <<<<<<<<<<<<<<
 *             return _chardev
 *         if S_ISBLK(mode):
 */
  }

  /* "bzrlib/_readdir_pyx.pyx":189
 *         if S_ISCHR(mode):
 *             return _chardev
 *         if S_ISBLK(mode):             # <<<<<<<<<<<<<<
 *             return _block
 *         if S_ISLNK(mode):
 */
  __pyx_t_1 = (S_ISBLK(__pyx_v_mode) != 0);
  if (__pyx_t_1) {

    /* "bzrlib/_readdir_pyx.pyx":190
 *             return _chardev
 *         if S_ISBLK(mode):
 *             return _block             # <<<<<<<<<<<<<<
 *         if S_ISLNK(mode):
 *             return _symlink
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_6bzrlib_12_readdir_pyx__block);
    __pyx_r = __pyx_v_6bzrlib_12_readdir_pyx__block;
    goto __pyx_L0;

    /* "bzrlib/_readdir_pyx.pyx":189
 *         if S_ISCHR(mode):
 *             return _chardev
 *         if S_ISBLK(mode):             # <<<<<<<<<<<<<<
 *             return _block
 *         if S_ISLNK(mode):
 */
  }

  /* "bzrlib/_readdir_pyx.pyx":191
 *         if S_ISBLK(mode):
 *             return _block
 *         if S_ISLNK(mode):             # <<<<<<<<<<<<<<
 *             return _symlink
 *         if S_ISFIFO(mode):
 */
  __pyx_t_1 = (S_ISLNK(__pyx_v_mode) != 0);
  if (__pyx_t_1) {

    /* "bzrlib/_readdir_pyx.pyx":192
 *             return _block
 *         if S_ISLNK(mode):
 *             return _symlink             # <<<<<<<<<<<<<<
 *         if S_ISFIFO(mode):
 *             return _fifo
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_6bzrlib_12_readdir_pyx__symlink);
    __pyx_r = __pyx_v_6bzrlib_12_readdir_pyx__symlink;
    goto __pyx_L0;

    /* "bzrlib/_readdir_pyx.pyx":191
 *         if S_ISBLK(mode):
 *             return _block
 *         if S_ISLNK(mode):             # <<<<<<<<<<<<<<
 *             return _symlink
 *         if S_ISFIFO(mode):
 */
  }

  /* "bzrlib/_readdir_pyx.pyx":193
 *         if S_ISLNK(mode):
 *             return _symlink
 *         if S_ISFIFO(mode):             # <<<<<<<<<<<<<<
 *             return _fifo
 *         if S_ISSOCK(mode):
 */
  __pyx_t_1 = (S_ISFIFO(__pyx_v_mode) != 0);
  if (__pyx_t_1) {

    /* "bzrlib/_readdir_pyx.pyx":194
 *             return _symlink
 *         if S_ISFIFO(mode):
 *             return _fifo             # <<<<<<<<<<<<<<
 *         if S_ISSOCK(mode):
 *             return _socket
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_6bzrlib_12_readdir_pyx__fifo);
    __pyx_r = __pyx_v_6bzrlib_12_readdir_pyx__fifo;
    goto __pyx_L0;

    /* "bzrlib/_readdir_pyx.pyx":193
 *         if S_ISLNK(mode):
 *             return _symlink
 *         if S_ISFIFO(mode):             # <<<<<<<<<<<<<<
 *             return _fifo
 *         if S_ISSOCK(mode):
 */
  }

  /* "bzrlib/_readdir_pyx.pyx":195
 *         if S_ISFIFO(mode):
 *             return _fifo
 *         if S_ISSOCK(mode):             # <<<<<<<<<<<<<<
 *             return _socket
 *         return _unknown
 */
  __pyx_t_1 = (S_ISSOCK(__pyx_v_mode) != 0);
  if (__pyx_t_1) {

    /* "bzrlib/_readdir_pyx.pyx":196
 *             return _fifo
 *         if S_ISSOCK(mode):
 *             return _socket             # <<<<<<<<<<<<<<
 *         return _unknown
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_6bzrlib_12_readdir_pyx__socket);
    __pyx_r = __pyx_v_6bzrlib_12_readdir_pyx__socket;
    goto __pyx_L0;

    /* "bzrlib/_readdir_pyx.pyx":195
 *         if S_ISFIFO(mode):
 *             return _fifo
 *         if S_ISSOCK(mode):             # <<<<<<<<<<<<<<
 *             return _socket
 *         return _unknown
 */
  }

  /* "bzrlib/_readdir_pyx.pyx":197
 *         if S_ISSOCK(mode):
 *             return _socket
 *         return _unknown             # <<<<<<<<<<<<<<
 * 
 *     def top_prefix_to_starting_dir(self, top, prefix=""):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_6bzrlib_12_readdir_pyx__unknown);
  __pyx_r = __pyx_v_6bzrlib_12_readdir_pyx__unknown;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":181
 *         return self._kind_from_mode(mode)
 * 
 *     cdef _kind_from_mode(self, int mode):             # <<<<<<<<<<<<<<
 *         # Files and directories are the most common - check them first.
 *         if S_ISREG(mode):
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":199
 *         return _unknown
 * 
 *     def top_prefix_to_starting_dir(self, top, prefix=""):             # <<<<<<<<<<<<<<
 *         """See DirReader.top_prefix_to_starting_dir."""
 *         return (_safe_utf8(prefix), None, None, None, _safe_utf8(top))
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_13UTF8DirReader_3top_prefix_to_starting_dir(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6bzrlib_12_readdir_pyx_13UTF8DirReader_2top_prefix_to_starting_dir[] = "See DirReader.top_prefix_to_starting_dir.";
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_13UTF8DirReader_3top_prefix_to_starting_dir(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_top = 0;
  PyObject *__pyx_v_prefix = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("top_prefix_to_starting_dir (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_top,&__pyx_n_s_prefix,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)__pyx_kp_s_);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_top)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_prefix);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "top_prefix_to_starting_dir") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_top = values[0];
    __pyx_v_prefix = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("top_prefix_to_starting_dir", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 199; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzrlib._readdir_pyx.UTF8DirReader.top_prefix_to_starting_dir", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzrlib_12_readdir_pyx_13UTF8DirReader_2top_prefix_to_starting_dir(((struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *)__pyx_v_self), __pyx_v_top, __pyx_v_prefix);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_13UTF8DirReader_2top_prefix_to_starting_dir(CYTHON_UNUSED struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *__pyx_v_self, PyObject *__pyx_v_top, PyObject *__pyx_v_prefix) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("top_prefix_to_starting_dir", 0);

  /* "bzrlib/_readdir_pyx.pyx":201
 *     def top_prefix_to_starting_dir(self, top, prefix=""):
 *         """See DirReader.top_prefix_to_starting_dir."""
 *         return (_safe_utf8(prefix), None, None, None, _safe_utf8(top))             # <<<<<<<<<<<<<<
 * 
 *     def read_dir(self, prefix, top):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_6bzrlib_12_readdir_pyx__safe_utf8);
  __pyx_t_2 = __pyx_v_6bzrlib_12_readdir_pyx__safe_utf8; __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_prefix); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_prefix);
    __Pyx_GIVEREF(__pyx_v_prefix);
    PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_prefix);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_INCREF(__pyx_v_6bzrlib_12_readdir_pyx__safe_utf8);
  __pyx_t_4 = __pyx_v_6bzrlib_12_readdir_pyx__safe_utf8; __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_top); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_top);
    __Pyx_GIVEREF(__pyx_v_top);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_top);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 3, Py_None);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":199
 *         return _unknown
 * 
 *     def top_prefix_to_starting_dir(self, top, prefix=""):             # <<<<<<<<<<<<<<
 *         """See DirReader.top_prefix_to_starting_dir."""
 *         return (_safe_utf8(prefix), None, None, None, _safe_utf8(top))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("bzrlib._readdir_pyx.UTF8DirReader.top_prefix_to_starting_dir", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":203
 *         return (_safe_utf8(prefix), None, None, None, _safe_utf8(top))
 * 
 *     def read_dir(self, prefix, top):             # <<<<<<<<<<<<<<
 *         """Read a single directory from a utf8 file system.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_13UTF8DirReader_5read_dir(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_6bzrlib_12_readdir_pyx_13UTF8DirReader_4read_dir[] = "Read a single directory from a utf8 file system.\n\n        All paths in and out are utf8.\n\n        This sub-function is called when we know the filesystem is already in utf8\n        encoding. So we don't need to transcode filenames.\n\n        See DirReader.read_dir for details.\n        ";
static PyObject *__pyx_pw_6bzrlib_12_readdir_pyx_13UTF8DirReader_5read_dir(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_top = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("read_dir (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_prefix,&__pyx_n_s_top,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_prefix)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_top)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("read_dir", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 203; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "read_dir") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 203; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_prefix = values[0];
    __pyx_v_top = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("read_dir", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 203; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("bzrlib._readdir_pyx.UTF8DirReader.read_dir", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6bzrlib_12_readdir_pyx_13UTF8DirReader_4read_dir(((struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *)__pyx_v_self), __pyx_v_prefix, __pyx_v_top);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6bzrlib_12_readdir_pyx_13UTF8DirReader_4read_dir(struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *__pyx_v_self, PyObject *__pyx_v_prefix, PyObject *__pyx_v_top) {
  PyObject *__pyx_v_newval = 0;
  int __pyx_v_index;
  int __pyx_v_length;
  void *__pyx_v_atuple;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_new_val_obj;
  PyObject *__pyx_v_relprefix = NULL;
  PyObject *__pyx_v_top_slash = NULL;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  void *__pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("read_dir", 0);

  /* "bzrlib/_readdir_pyx.pyx":223
 *         cdef PyObject * new_val_obj
 * 
 *         if PyString_Size(prefix):             # <<<<<<<<<<<<<<
 *             relprefix = prefix + '/'
 *         else:
 */
  __pyx_t_1 = (PyString_Size(__pyx_v_prefix) != 0);
  if (__pyx_t_1) {

    /* "bzrlib/_readdir_pyx.pyx":224
 * 
 *         if PyString_Size(prefix):
 *             relprefix = prefix + '/'             # <<<<<<<<<<<<<<
 *         else:
 *             relprefix = ''
 */
    __pyx_t_2 = PyNumber_Add(__pyx_v_prefix, __pyx_kp_s__2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 224; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_relprefix = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "bzrlib/_readdir_pyx.pyx":223
 *         cdef PyObject * new_val_obj
 * 
 *         if PyString_Size(prefix):             # <<<<<<<<<<<<<<
 *             relprefix = prefix + '/'
 *         else:
 */
    goto __pyx_L3;
  }

  /* "bzrlib/_readdir_pyx.pyx":226
 *             relprefix = prefix + '/'
 *         else:
 *             relprefix = ''             # <<<<<<<<<<<<<<
 *         top_slash = top + '/'
 * 
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_kp_s_);
    __pyx_v_relprefix = __pyx_kp_s_;
  }
  __pyx_L3:;

  /* "bzrlib/_readdir_pyx.pyx":227
 *         else:
 *             relprefix = ''
 *         top_slash = top + '/'             # <<<<<<<<<<<<<<
 * 
 *         # read_dir supplies in should-stat order.
 */
  __pyx_t_2 = PyNumber_Add(__pyx_v_top, __pyx_kp_s__2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 227; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_top_slash = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "bzrlib/_readdir_pyx.pyx":231
 *         # read_dir supplies in should-stat order.
 *         # for _, name in sorted(_listdir(top)):
 *         result = _read_dir(top)             # <<<<<<<<<<<<<<
 *         length = len(result)
 *         # result.sort()
 */
  __pyx_t_2 = __pyx_f_6bzrlib_12_readdir_pyx__read_dir(__pyx_v_top); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "bzrlib/_readdir_pyx.pyx":232
 *         # for _, name in sorted(_listdir(top)):
 *         result = _read_dir(top)
 *         length = len(result)             # <<<<<<<<<<<<<<
 *         # result.sort()
 *         for index from 0 <= index < length:
 */
  __pyx_t_3 = PyObject_Length(__pyx_v_result); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_length = __pyx_t_3;

  /* "bzrlib/_readdir_pyx.pyx":234
 *         length = len(result)
 *         # result.sort()
 *         for index from 0 <= index < length:             # <<<<<<<<<<<<<<
 *             atuple = PyList_GetItem_object_void(result, index)
 *             name = <object>PyTuple_GetItem_void_void(atuple, 1)
 */
  __pyx_t_4 = __pyx_v_length;
  for (__pyx_v_index = 0; __pyx_v_index < __pyx_t_4; __pyx_v_index++) {

    /* "bzrlib/_readdir_pyx.pyx":235
 *         # result.sort()
 *         for index from 0 <= index < length:
 *             atuple = PyList_GetItem_object_void(result, index)             # <<<<<<<<<<<<<<
 *             name = <object>PyTuple_GetItem_void_void(atuple, 1)
 *             # We have a tuple with (inode, name, None, statvalue, None)
 */
    __pyx_v_atuple = PyList_GET_ITEM(__pyx_v_result, __pyx_v_index);

    /* "bzrlib/_readdir_pyx.pyx":236
 *         for index from 0 <= index < length:
 *             atuple = PyList_GetItem_object_void(result, index)
 *             name = <object>PyTuple_GetItem_void_void(atuple, 1)             # <<<<<<<<<<<<<<
 *             # We have a tuple with (inode, name, None, statvalue, None)
 *             # Now edit it:
 */
    __pyx_t_5 = PyTuple_GET_ITEM(__pyx_v_atuple, 1);
    __pyx_t_2 = ((PyObject *)__pyx_t_5);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "bzrlib/_readdir_pyx.pyx":241
 *             # inode -> path_from_top
 *             # direct concat - faster than operator +.
 *             new_val_obj = <PyObject *>relprefix             # <<<<<<<<<<<<<<
 *             Py_INCREF(relprefix)
 *             PyString_Concat(&new_val_obj, name)
 */
    __pyx_v_new_val_obj = ((PyObject *)__pyx_v_relprefix);

    /* "bzrlib/_readdir_pyx.pyx":242
 *             # direct concat - faster than operator +.
 *             new_val_obj = <PyObject *>relprefix
 *             Py_INCREF(relprefix)             # <<<<<<<<<<<<<<
 *             PyString_Concat(&new_val_obj, name)
 *             if NULL == new_val_obj:
 */
    Py_INCREF(__pyx_v_relprefix);

    /* "bzrlib/_readdir_pyx.pyx":243
 *             new_val_obj = <PyObject *>relprefix
 *             Py_INCREF(relprefix)
 *             PyString_Concat(&new_val_obj, name)             # <<<<<<<<<<<<<<
 *             if NULL == new_val_obj:
 *                 # PyString_Concat will have setup an exception, but how to get
 */
    PyString_Concat((&__pyx_v_new_val_obj), __pyx_v_name);

    /* "bzrlib/_readdir_pyx.pyx":244
 *             Py_INCREF(relprefix)
 *             PyString_Concat(&new_val_obj, name)
 *             if NULL == new_val_obj:             # <<<<<<<<<<<<<<
 *                 # PyString_Concat will have setup an exception, but how to get
 *                 # at it?
 */
    __pyx_t_1 = ((NULL == __pyx_v_new_val_obj) != 0);
    if (__pyx_t_1) {

      /* "bzrlib/_readdir_pyx.pyx":247
 *                 # PyString_Concat will have setup an exception, but how to get
 *                 # at it?
 *                 raise Exception("failed to strcat")             # <<<<<<<<<<<<<<
 *             PyTuple_SetItem_obj(atuple, 0, new_val_obj)
 *             # 1st None -> kind
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_Exception, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "bzrlib/_readdir_pyx.pyx":244
 *             Py_INCREF(relprefix)
 *             PyString_Concat(&new_val_obj, name)
 *             if NULL == new_val_obj:             # <<<<<<<<<<<<<<
 *                 # PyString_Concat will have setup an exception, but how to get
 *                 # at it?
 */
    }

    /* "bzrlib/_readdir_pyx.pyx":248
 *                 # at it?
 *                 raise Exception("failed to strcat")
 *             PyTuple_SetItem_obj(atuple, 0, new_val_obj)             # <<<<<<<<<<<<<<
 *             # 1st None -> kind
 *             newval = self._kind_from_mode(
 */
    __pyx_t_6 = PyTuple_SetItem(__pyx_v_atuple, 0, __pyx_v_new_val_obj); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 248; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "bzrlib/_readdir_pyx.pyx":251
 *             # 1st None -> kind
 *             newval = self._kind_from_mode(
 *                 (<_Stat>PyTuple_GetItem_void_void(atuple, 3)).st_mode)             # <<<<<<<<<<<<<<
 *             Py_INCREF(newval)
 *             PyTuple_SetItem(atuple, 2, newval)
 */
    __pyx_t_5 = PyTuple_GET_ITEM(__pyx_v_atuple, 3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_t_5), __pyx_n_s_st_mode); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 251; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 251; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "bzrlib/_readdir_pyx.pyx":250
 *             PyTuple_SetItem_obj(atuple, 0, new_val_obj)
 *             # 1st None -> kind
 *             newval = self._kind_from_mode(             # <<<<<<<<<<<<<<
 *                 (<_Stat>PyTuple_GetItem_void_void(atuple, 3)).st_mode)
 *             Py_INCREF(newval)
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_6bzrlib_12_readdir_pyx_UTF8DirReader *)__pyx_v_self->__pyx_vtab)->_kind_from_mode(__pyx_v_self, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 250; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_newval, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "bzrlib/_readdir_pyx.pyx":252
 *             newval = self._kind_from_mode(
 *                 (<_Stat>PyTuple_GetItem_void_void(atuple, 3)).st_mode)
 *             Py_INCREF(newval)             # <<<<<<<<<<<<<<
 *             PyTuple_SetItem(atuple, 2, newval)
 *             # 2nd None -> abspath # for all - the caller may need to stat files
 */
    Py_INCREF(__pyx_v_newval);

    /* "bzrlib/_readdir_pyx.pyx":253
 *                 (<_Stat>PyTuple_GetItem_void_void(atuple, 3)).st_mode)
 *             Py_INCREF(newval)
 *             PyTuple_SetItem(atuple, 2, newval)             # <<<<<<<<<<<<<<
 *             # 2nd None -> abspath # for all - the caller may need to stat files
 *             # etc.
 */
    __pyx_t_6 = PyTuple_SetItem(__pyx_v_atuple, 2, __pyx_v_newval); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 253; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "bzrlib/_readdir_pyx.pyx":257
 *             # etc.
 *             # direct concat - faster than operator +.
 *             new_val_obj = <PyObject *>top_slash             # <<<<<<<<<<<<<<
 *             Py_INCREF(top_slash)
 *             PyString_Concat(&new_val_obj, name)
 */
    __pyx_v_new_val_obj = ((PyObject *)__pyx_v_top_slash);

    /* "bzrlib/_readdir_pyx.pyx":258
 *             # direct concat - faster than operator +.
 *             new_val_obj = <PyObject *>top_slash
 *             Py_INCREF(top_slash)             # <<<<<<<<<<<<<<
 *             PyString_Concat(&new_val_obj, name)
 *             if NULL == new_val_obj:
 */
    Py_INCREF(__pyx_v_top_slash);

    /* "bzrlib/_readdir_pyx.pyx":259
 *             new_val_obj = <PyObject *>top_slash
 *             Py_INCREF(top_slash)
 *             PyString_Concat(&new_val_obj, name)             # <<<<<<<<<<<<<<
 *             if NULL == new_val_obj:
 *                 # PyString_Concat will have setup an exception, but how to get
 */
    PyString_Concat((&__pyx_v_new_val_obj), __pyx_v_name);

    /* "bzrlib/_readdir_pyx.pyx":260
 *             Py_INCREF(top_slash)
 *             PyString_Concat(&new_val_obj, name)
 *             if NULL == new_val_obj:             # <<<<<<<<<<<<<<
 *                 # PyString_Concat will have setup an exception, but how to get
 *                 # at it?
 */
    __pyx_t_1 = ((NULL == __pyx_v_new_val_obj) != 0);
    if (__pyx_t_1) {

      /* "bzrlib/_readdir_pyx.pyx":263
 *                 # PyString_Concat will have setup an exception, but how to get
 *                 # at it?
 *                 raise Exception("failed to strcat")             # <<<<<<<<<<<<<<
 *             PyTuple_SetItem_obj(atuple, 4, new_val_obj)
 *         return result
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_Exception, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "bzrlib/_readdir_pyx.pyx":260
 *             Py_INCREF(top_slash)
 *             PyString_Concat(&new_val_obj, name)
 *             if NULL == new_val_obj:             # <<<<<<<<<<<<<<
 *                 # PyString_Concat will have setup an exception, but how to get
 *                 # at it?
 */
    }

    /* "bzrlib/_readdir_pyx.pyx":264
 *                 # at it?
 *                 raise Exception("failed to strcat")
 *             PyTuple_SetItem_obj(atuple, 4, new_val_obj)             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
    __pyx_t_6 = PyTuple_SetItem(__pyx_v_atuple, 4, __pyx_v_new_val_obj); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "bzrlib/_readdir_pyx.pyx":265
 *                 raise Exception("failed to strcat")
 *             PyTuple_SetItem_obj(atuple, 4, new_val_obj)
 *         return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":203
 *         return (_safe_utf8(prefix), None, None, None, _safe_utf8(top))
 * 
 *     def read_dir(self, prefix, top):             # <<<<<<<<<<<<<<
 *         """Read a single directory from a utf8 file system.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("bzrlib._readdir_pyx.UTF8DirReader.read_dir", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_newval);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XDECREF(__pyx_v_relprefix);
  __Pyx_XDECREF(__pyx_v_top_slash);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":268
 * 
 * 
 * cdef raise_os_error(int errnum, char *msg_prefix, path):             # <<<<<<<<<<<<<<
 *     if errnum == EINTR:
 *         PyErr_CheckSignals()
 */

static PyObject *__pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(int __pyx_v_errnum, char *__pyx_v_msg_prefix, PyObject *__pyx_v_path) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("raise_os_error", 0);

  /* "bzrlib/_readdir_pyx.pyx":269
 * 
 * cdef raise_os_error(int errnum, char *msg_prefix, path):
 *     if errnum == EINTR:             # <<<<<<<<<<<<<<
 *         PyErr_CheckSignals()
 *     raise OSError(errnum, msg_prefix + strerror(errnum), path)
 */
  __pyx_t_1 = ((__pyx_v_errnum == EINTR) != 0);
  if (__pyx_t_1) {

    /* "bzrlib/_readdir_pyx.pyx":270
 * cdef raise_os_error(int errnum, char *msg_prefix, path):
 *     if errnum == EINTR:
 *         PyErr_CheckSignals()             # <<<<<<<<<<<<<<
 *     raise OSError(errnum, msg_prefix + strerror(errnum), path)
 * 
 */
    __pyx_t_2 = PyErr_CheckSignals(); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 270; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "bzrlib/_readdir_pyx.pyx":269
 * 
 * cdef raise_os_error(int errnum, char *msg_prefix, path):
 *     if errnum == EINTR:             # <<<<<<<<<<<<<<
 *         PyErr_CheckSignals()
 *     raise OSError(errnum, msg_prefix + strerror(errnum), path)
 */
  }

  /* "bzrlib/_readdir_pyx.pyx":271
 *     if errnum == EINTR:
 *         PyErr_CheckSignals()
 *     raise OSError(errnum, msg_prefix + strerror(errnum), path)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_errnum); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_msg_prefix); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBytes_FromString(strerror(__pyx_v_errnum)); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyNumber_Add(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
  __Pyx_INCREF(__pyx_v_path);
  __Pyx_GIVEREF(__pyx_v_path);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_path);
  __pyx_t_3 = 0;
  __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_OSError, __pyx_t_5, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_Raise(__pyx_t_6, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "bzrlib/_readdir_pyx.pyx":268
 * 
 * 
 * cdef raise_os_error(int errnum, char *msg_prefix, path):             # <<<<<<<<<<<<<<
 *     if errnum == EINTR:
 *         PyErr_CheckSignals()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("bzrlib._readdir_pyx.raise_os_error", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "bzrlib/_readdir_pyx.pyx":274
 * 
 * 
 * cdef _read_dir(path):             # <<<<<<<<<<<<<<
 *     """Like os.listdir, this reads the contents of a directory.
 * 
 */

static PyObject *__pyx_f_6bzrlib_12_readdir_pyx__read_dir(PyObject *__pyx_v_path) {
  DIR *__pyx_v_the_dir;
  dirent *__pyx_v_entry;
  dirent __pyx_v_sentinel;
  char *__pyx_v_name;
  int __pyx_v_stat_result;
  struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *__pyx_v_statvalue = 0;
  int __pyx_v_orig_dir_fd;
  PyObject *__pyx_v_result = NULL;
  int __pyx_v_failed;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  char const *__pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  char const *__pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_read_dir", 0);

  /* "bzrlib/_readdir_pyx.pyx":295
 *     # Avoid chdir('') because it causes problems on Sun OS, and avoid this if
 *     # staying in .
 *     if path != "" and path != '.':             # <<<<<<<<<<<<<<
 *         # we change into the requested directory before reading, and back at the
 *         # end, because that turns out to make the stat calls measurably faster than
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_path, __pyx_kp_s_, Py_NE)); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_path, __pyx_kp_s__5, Py_NE)); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 295; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "bzrlib/_readdir_pyx.pyx":299
 *         # end, because that turns out to make the stat calls measurably faster than
 *         # passing full paths every time.
 *         orig_dir_fd = open(".", O_RDONLY, 0)             # <<<<<<<<<<<<<<
 *         if orig_dir_fd == -1:
 *             raise_os_error(errno, "open: ", ".")
 */
    __pyx_v_orig_dir_fd = open(__pyx_k__5, O_RDONLY, 0);

    /* "bzrlib/_readdir_pyx.pyx":300
 *         # passing full paths every time.
 *         orig_dir_fd = open(".", O_RDONLY, 0)
 *         if orig_dir_fd == -1:             # <<<<<<<<<<<<<<
 *             raise_os_error(errno, "open: ", ".")
 *         if -1 == chdir(path):
 */
    __pyx_t_1 = ((__pyx_v_orig_dir_fd == -1L) != 0);
    if (__pyx_t_1) {

      /* "bzrlib/_readdir_pyx.pyx":301
 *         orig_dir_fd = open(".", O_RDONLY, 0)
 *         if orig_dir_fd == -1:
 *             raise_os_error(errno, "open: ", ".")             # <<<<<<<<<<<<<<
 *         if -1 == chdir(path):
 *             # Ignore the return value, because we are already raising an
 */
      __pyx_t_3 = __pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(errno, __pyx_k_open, __pyx_kp_s__5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "bzrlib/_readdir_pyx.pyx":300
 *         # passing full paths every time.
 *         orig_dir_fd = open(".", O_RDONLY, 0)
 *         if orig_dir_fd == -1:             # <<<<<<<<<<<<<<
 *             raise_os_error(errno, "open: ", ".")
 *         if -1 == chdir(path):
 */
    }

    /* "bzrlib/_readdir_pyx.pyx":302
 *         if orig_dir_fd == -1:
 *             raise_os_error(errno, "open: ", ".")
 *         if -1 == chdir(path):             # <<<<<<<<<<<<<<
 *             # Ignore the return value, because we are already raising an
 *             # exception
 */
    __pyx_t_4 = __Pyx_PyObject_AsString(__pyx_v_path); if (unlikely((!__pyx_t_4) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_1 = ((-1L == chdir(__pyx_t_4)) != 0);
    if (__pyx_t_1) {

      /* "bzrlib/_readdir_pyx.pyx":305
 *             # Ignore the return value, because we are already raising an
 *             # exception
 *             close(orig_dir_fd)             # <<<<<<<<<<<<<<
 *             raise_os_error(errno, "chdir: ", path)
 *     else:
 */
      close(__pyx_v_orig_dir_fd);

      /* "bzrlib/_readdir_pyx.pyx":306
 *             # exception
 *             close(orig_dir_fd)
 *             raise_os_error(errno, "chdir: ", path)             # <<<<<<<<<<<<<<
 *     else:
 *         orig_dir_fd = -1
 */
      __pyx_t_3 = __pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(errno, __pyx_k_chdir, __pyx_v_path); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 306; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "bzrlib/_readdir_pyx.pyx":302
 *         if orig_dir_fd == -1:
 *             raise_os_error(errno, "open: ", ".")
 *         if -1 == chdir(path):             # <<<<<<<<<<<<<<
 *             # Ignore the return value, because we are already raising an
 *             # exception
 */
    }

    /* "bzrlib/_readdir_pyx.pyx":295
 *     # Avoid chdir('') because it causes problems on Sun OS, and avoid this if
 *     # staying in .
 *     if path != "" and path != '.':             # <<<<<<<<<<<<<<
 *         # we change into the requested directory before reading, and back at the
 *         # end, because that turns out to make the stat calls measurably faster than
 */
    goto __pyx_L3;
  }

  /* "bzrlib/_readdir_pyx.pyx":308
 *             raise_os_error(errno, "chdir: ", path)
 *     else:
 *         orig_dir_fd = -1             # <<<<<<<<<<<<<<
 * 
 *     try:
 */
  /*else*/ {
    __pyx_v_orig_dir_fd = -1;
  }
  __pyx_L3:;

  /* "bzrlib/_readdir_pyx.pyx":310
 *         orig_dir_fd = -1
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         the_dir = opendir(".")
 *         if NULL == the_dir:
 */
  /*try:*/ {

    /* "bzrlib/_readdir_pyx.pyx":311
 * 
 *     try:
 *         the_dir = opendir(".")             # <<<<<<<<<<<<<<
 *         if NULL == the_dir:
 *             raise_os_error(errno, "opendir: ", path)
 */
    __pyx_v_the_dir = opendir(__pyx_k__5);

    /* "bzrlib/_readdir_pyx.pyx":312
 *     try:
 *         the_dir = opendir(".")
 *         if NULL == the_dir:             # <<<<<<<<<<<<<<
 *             raise_os_error(errno, "opendir: ", path)
 *         try:
 */
    __pyx_t_1 = ((NULL == __pyx_v_the_dir) != 0);
    if (__pyx_t_1) {

      /* "bzrlib/_readdir_pyx.pyx":313
 *         the_dir = opendir(".")
 *         if NULL == the_dir:
 *             raise_os_error(errno, "opendir: ", path)             # <<<<<<<<<<<<<<
 *         try:
 *             result = []
 */
      __pyx_t_3 = __pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(errno, __pyx_k_opendir, __pyx_v_path); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 313; __pyx_clineno = __LINE__; goto __pyx_L9_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "bzrlib/_readdir_pyx.pyx":312
 *     try:
 *         the_dir = opendir(".")
 *         if NULL == the_dir:             # <<<<<<<<<<<<<<
 *             raise_os_error(errno, "opendir: ", path)
 *         try:
 */
    }

    /* "bzrlib/_readdir_pyx.pyx":314
 *         if NULL == the_dir:
 *             raise_os_error(errno, "opendir: ", path)
 *         try:             # <<<<<<<<<<<<<<
 *             result = []
 *             entry = &sentinel
 */
    /*try:*/ {

      /* "bzrlib/_readdir_pyx.pyx":315
 *             raise_os_error(errno, "opendir: ", path)
 *         try:
 *             result = []             # <<<<<<<<<<<<<<
 *             entry = &sentinel
 *             while entry != NULL:
 */
      __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 315; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_v_result = ((PyObject*)__pyx_t_3);
      __pyx_t_3 = 0;

      /* "bzrlib/_readdir_pyx.pyx":316
 *         try:
 *             result = []
 *             entry = &sentinel             # <<<<<<<<<<<<<<
 *             while entry != NULL:
 *                 # Unlike most libc functions, readdir needs errno set to 0
 */
      __pyx_v_entry = (&__pyx_v_sentinel);

      /* "bzrlib/_readdir_pyx.pyx":317
 *             result = []
 *             entry = &sentinel
 *             while entry != NULL:             # <<<<<<<<<<<<<<
 *                 # Unlike most libc functions, readdir needs errno set to 0
 *                 # beforehand so that eof can be distinguished from errors.  See
 */
      while (1) {
        __pyx_t_1 = ((__pyx_v_entry != NULL) != 0);
        if (!__pyx_t_1) break;

        /* "bzrlib/_readdir_pyx.pyx":321
 *                 # beforehand so that eof can be distinguished from errors.  See
 *                 # <https://bugs.launchpad.net/bzr/+bug/279381>
 *                 while True:             # <<<<<<<<<<<<<<
 *                     errno = 0
 *                     entry = readdir(the_dir)
 */
        while (1) {

          /* "bzrlib/_readdir_pyx.pyx":322
 *                 # <https://bugs.launchpad.net/bzr/+bug/279381>
 *                 while True:
 *                     errno = 0             # <<<<<<<<<<<<<<
 *                     entry = readdir(the_dir)
 *                     if entry == NULL and (errno == EAGAIN or errno == EINTR):
 */
          errno = 0;

          /* "bzrlib/_readdir_pyx.pyx":323
 *                 while True:
 *                     errno = 0
 *                     entry = readdir(the_dir)             # <<<<<<<<<<<<<<
 *                     if entry == NULL and (errno == EAGAIN or errno == EINTR):
 *                         if errno == EINTR:
 */
          __pyx_v_entry = readdir(__pyx_v_the_dir);

          /* "bzrlib/_readdir_pyx.pyx":324
 *                     errno = 0
 *                     entry = readdir(the_dir)
 *                     if entry == NULL and (errno == EAGAIN or errno == EINTR):             # <<<<<<<<<<<<<<
 *                         if errno == EINTR:
 *                             PyErr_CheckSignals()
 */
          __pyx_t_2 = ((__pyx_v_entry == NULL) != 0);
          if (__pyx_t_2) {
          } else {
            __pyx_t_1 = __pyx_t_2;
            goto __pyx_L20_bool_binop_done;
          }
          __pyx_t_2 = ((errno == EAGAIN) != 0);
          if (!__pyx_t_2) {
          } else {
            __pyx_t_1 = __pyx_t_2;
            goto __pyx_L20_bool_binop_done;
          }
          __pyx_t_2 = ((errno == EINTR) != 0);
          __pyx_t_1 = __pyx_t_2;
          __pyx_L20_bool_binop_done:;
          if (__pyx_t_1) {

            /* "bzrlib/_readdir_pyx.pyx":325
 *                     entry = readdir(the_dir)
 *                     if entry == NULL and (errno == EAGAIN or errno == EINTR):
 *                         if errno == EINTR:             # <<<<<<<<<<<<<<
 *                             PyErr_CheckSignals()
 *                         # try again
 */
            __pyx_t_1 = ((errno == EINTR) != 0);
            if (__pyx_t_1) {

              /* "bzrlib/_readdir_pyx.pyx":326
 *                     if entry == NULL and (errno == EAGAIN or errno == EINTR):
 *                         if errno == EINTR:
 *                             PyErr_CheckSignals()             # <<<<<<<<<<<<<<
 *                         # try again
 *                         continue
 */
              __pyx_t_5 = PyErr_CheckSignals(); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 326; __pyx_clineno = __LINE__; goto __pyx_L13_error;}

              /* "bzrlib/_readdir_pyx.pyx":325
 *                     entry = readdir(the_dir)
 *                     if entry == NULL and (errno == EAGAIN or errno == EINTR):
 *                         if errno == EINTR:             # <<<<<<<<<<<<<<
 *                             PyErr_CheckSignals()
 *                         # try again
 */
            }

            /* "bzrlib/_readdir_pyx.pyx":328
 *                             PyErr_CheckSignals()
 *                         # try again
 *                         continue             # <<<<<<<<<<<<<<
 *                     else:
 *                         break
 */
            goto __pyx_L17_continue;

            /* "bzrlib/_readdir_pyx.pyx":324
 *                     errno = 0
 *                     entry = readdir(the_dir)
 *                     if entry == NULL and (errno == EAGAIN or errno == EINTR):             # <<<<<<<<<<<<<<
 *                         if errno == EINTR:
 *                             PyErr_CheckSignals()
 */
          }

          /* "bzrlib/_readdir_pyx.pyx":330
 *                         continue
 *                     else:
 *                         break             # <<<<<<<<<<<<<<
 *                 if entry == NULL:
 *                     if errno == ENOTDIR or errno == 0:
 */
          /*else*/ {
            goto __pyx_L18_break;
          }
          __pyx_L17_continue:;
        }
        __pyx_L18_break:;

        /* "bzrlib/_readdir_pyx.pyx":331
 *                     else:
 *                         break
 *                 if entry == NULL:             # <<<<<<<<<<<<<<
 *                     if errno == ENOTDIR or errno == 0:
 *                         # We see ENOTDIR at the end of a normal directory.
 */
        __pyx_t_1 = ((__pyx_v_entry == NULL) != 0);
        if (__pyx_t_1) {

          /* "bzrlib/_readdir_pyx.pyx":332
 *                         break
 *                 if entry == NULL:
 *                     if errno == ENOTDIR or errno == 0:             # <<<<<<<<<<<<<<
 *                         # We see ENOTDIR at the end of a normal directory.
 *                         # As ENOTDIR for read_dir(file) is triggered on opendir,
 */
          __pyx_t_2 = ((errno == ENOTDIR) != 0);
          if (!__pyx_t_2) {
          } else {
            __pyx_t_1 = __pyx_t_2;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_2 = ((errno == 0) != 0);
          __pyx_t_1 = __pyx_t_2;
          __pyx_L26_bool_binop_done:;
          if (__pyx_t_1) {

            /* "bzrlib/_readdir_pyx.pyx":336
 *                         # As ENOTDIR for read_dir(file) is triggered on opendir,
 *                         # we consider ENOTDIR to be 'no error'.
 *                         continue             # <<<<<<<<<<<<<<
 *                     else:
 *                         raise_os_error(errno, "readdir: ", path)
 */
            goto __pyx_L15_continue;

            /* "bzrlib/_readdir_pyx.pyx":332
 *                         break
 *                 if entry == NULL:
 *                     if errno == ENOTDIR or errno == 0:             # <<<<<<<<<<<<<<
 *                         # We see ENOTDIR at the end of a normal directory.
 *                         # As ENOTDIR for read_dir(file) is triggered on opendir,
 */
          }

          /* "bzrlib/_readdir_pyx.pyx":338
 *                         continue
 *                     else:
 *                         raise_os_error(errno, "readdir: ", path)             # <<<<<<<<<<<<<<
 *                 name = entry.d_name
 *                 if not (name[0] == c"." and (
 */
          /*else*/ {
            __pyx_t_3 = __pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(errno, __pyx_k_readdir, __pyx_v_path); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 338; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          }

          /* "bzrlib/_readdir_pyx.pyx":331
 *                     else:
 *                         break
 *                 if entry == NULL:             # <<<<<<<<<<<<<<
 *                     if errno == ENOTDIR or errno == 0:
 *                         # We see ENOTDIR at the end of a normal directory.
 */
        }

        /* "bzrlib/_readdir_pyx.pyx":339
 *                     else:
 *                         raise_os_error(errno, "readdir: ", path)
 *                 name = entry.d_name             # <<<<<<<<<<<<<<
 *                 if not (name[0] == c"." and (
 *                     (name[1] == 0) or
 */
        __pyx_t_4 = __pyx_v_entry->d_name;
        __pyx_v_name = __pyx_t_4;

        /* "bzrlib/_readdir_pyx.pyx":340
 *                         raise_os_error(errno, "readdir: ", path)
 *                 name = entry.d_name
 *                 if not (name[0] == c"." and (             # <<<<<<<<<<<<<<
 *                     (name[1] == 0) or
 *                     (name[1] == c"." and name[2] == 0))
 */
        __pyx_t_2 = (((__pyx_v_name[0]) == '.') != 0);
        if (__pyx_t_2) {
        } else {
          __pyx_t_1 = __pyx_t_2;
          goto __pyx_L29_bool_binop_done;
        }

        /* "bzrlib/_readdir_pyx.pyx":341
 *                 name = entry.d_name
 *                 if not (name[0] == c"." and (
 *                     (name[1] == 0) or             # <<<<<<<<<<<<<<
 *                     (name[1] == c"." and name[2] == 0))
 *                     ):
 */
        __pyx_t_2 = (((__pyx_v_name[1]) == 0) != 0);
        if (!__pyx_t_2) {
        } else {
          __pyx_t_1 = __pyx_t_2;
          goto __pyx_L29_bool_binop_done;
        }

        /* "bzrlib/_readdir_pyx.pyx":342
 *                 if not (name[0] == c"." and (
 *                     (name[1] == 0) or
 *                     (name[1] == c"." and name[2] == 0))             # <<<<<<<<<<<<<<
 *                     ):
 *                     statvalue = _Stat()
 */
        __pyx_t_2 = (((__pyx_v_name[1]) == '.') != 0);
        if (__pyx_t_2) {
        } else {
          __pyx_t_1 = __pyx_t_2;
          goto __pyx_L29_bool_binop_done;
        }
        __pyx_t_2 = (((__pyx_v_name[2]) == 0) != 0);
        __pyx_t_1 = __pyx_t_2;
        __pyx_L29_bool_binop_done:;

        /* "bzrlib/_readdir_pyx.pyx":340
 *                         raise_os_error(errno, "readdir: ", path)
 *                 name = entry.d_name
 *                 if not (name[0] == c"." and (             # <<<<<<<<<<<<<<
 *                     (name[1] == 0) or
 *                     (name[1] == c"." and name[2] == 0))
 */
        __pyx_t_2 = ((!__pyx_t_1) != 0);
        if (__pyx_t_2) {

          /* "bzrlib/_readdir_pyx.pyx":344
 *                     (name[1] == c"." and name[2] == 0))
 *                     ):
 *                     statvalue = _Stat()             # <<<<<<<<<<<<<<
 *                     stat_result = lstat(entry.d_name, &statvalue._st)
 *                     if stat_result != 0:
 */
          __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_6bzrlib_12_readdir_pyx__Stat), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 344; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_XDECREF_SET(__pyx_v_statvalue, ((struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat *)__pyx_t_3));
          __pyx_t_3 = 0;

          /* "bzrlib/_readdir_pyx.pyx":345
 *                     ):
 *                     statvalue = _Stat()
 *                     stat_result = lstat(entry.d_name, &statvalue._st)             # <<<<<<<<<<<<<<
 *                     if stat_result != 0:
 *                         if errno != ENOENT:
 */
          __pyx_v_stat_result = lstat(__pyx_v_entry->d_name, (&__pyx_v_statvalue->_st));

          /* "bzrlib/_readdir_pyx.pyx":346
 *                     statvalue = _Stat()
 *                     stat_result = lstat(entry.d_name, &statvalue._st)
 *                     if stat_result != 0:             # <<<<<<<<<<<<<<
 *                         if errno != ENOENT:
 *                             raise_os_error(errno, "lstat: ",
 */
          __pyx_t_2 = ((__pyx_v_stat_result != 0) != 0);
          if (__pyx_t_2) {

            /* "bzrlib/_readdir_pyx.pyx":347
 *                     stat_result = lstat(entry.d_name, &statvalue._st)
 *                     if stat_result != 0:
 *                         if errno != ENOENT:             # <<<<<<<<<<<<<<
 *                             raise_os_error(errno, "lstat: ",
 *                                 path + "/" + entry.d_name)
 */
            __pyx_t_2 = ((errno != ENOENT) != 0);
            if (__pyx_t_2) {

              /* "bzrlib/_readdir_pyx.pyx":349
 *                         if errno != ENOENT:
 *                             raise_os_error(errno, "lstat: ",
 *                                 path + "/" + entry.d_name)             # <<<<<<<<<<<<<<
 *                         else:
 *                             # the file seems to have disappeared after being
 */
              __pyx_t_3 = PyNumber_Add(__pyx_v_path, __pyx_kp_s__2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 349; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
              __Pyx_GOTREF(__pyx_t_3);
              __pyx_t_6 = __Pyx_PyObject_FromString(__pyx_v_entry->d_name); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 349; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
              __Pyx_GOTREF(__pyx_t_6);
              __pyx_t_7 = PyNumber_Add(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 349; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

              /* "bzrlib/_readdir_pyx.pyx":348
 *                     if stat_result != 0:
 *                         if errno != ENOENT:
 *                             raise_os_error(errno, "lstat: ",             # <<<<<<<<<<<<<<
 *                                 path + "/" + entry.d_name)
 *                         else:
 */
              __pyx_t_6 = __pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(errno, __pyx_k_lstat, __pyx_t_7); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 348; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
              __Pyx_GOTREF(__pyx_t_6);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

              /* "bzrlib/_readdir_pyx.pyx":347
 *                     stat_result = lstat(entry.d_name, &statvalue._st)
 *                     if stat_result != 0:
 *                         if errno != ENOENT:             # <<<<<<<<<<<<<<
 *                             raise_os_error(errno, "lstat: ",
 *                                 path + "/" + entry.d_name)
 */
              goto __pyx_L34;
            }

            /* "bzrlib/_readdir_pyx.pyx":354
 *                             # seen by readdir - perhaps a transient temporary
 *                             # file.  there's no point returning it.
 *                             continue             # <<<<<<<<<<<<<<
 *                     # We append a 5-tuple that can be modified in-place by the C
 *                     # api:
 */
            /*else*/ {
              goto __pyx_L15_continue;
            }
            __pyx_L34:;

            /* "bzrlib/_readdir_pyx.pyx":346
 *                     statvalue = _Stat()
 *                     stat_result = lstat(entry.d_name, &statvalue._st)
 *                     if stat_result != 0:             # <<<<<<<<<<<<<<
 *                         if errno != ENOENT:
 *                             raise_os_error(errno, "lstat: ",
 */
          }

          /* "bzrlib/_readdir_pyx.pyx":362
 *                     # statvalue (to keep)
 *                     # abspath (None, to set)
 *                     PyList_Append(result, (entry.d_ino, entry.d_name, None,             # <<<<<<<<<<<<<<
 *                         statvalue, None))
 *         finally:
 */
          __pyx_t_6 = __Pyx_PyInt_From_ino_t(__pyx_v_entry->d_ino); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_7 = __Pyx_PyObject_FromString(__pyx_v_entry->d_name); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
          __Pyx_GOTREF(__pyx_t_7);

          /* "bzrlib/_readdir_pyx.pyx":363
 *                     # abspath (None, to set)
 *                     PyList_Append(result, (entry.d_ino, entry.d_name, None,
 *                         statvalue, None))             # <<<<<<<<<<<<<<
 *         finally:
 *             if -1 == closedir(the_dir):
 */
          __pyx_t_3 = PyTuple_New(5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_7);
          __Pyx_INCREF(Py_None);
          __Pyx_GIVEREF(Py_None);
          PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
          __Pyx_INCREF(((PyObject *)__pyx_v_statvalue));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_statvalue));
          PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_statvalue));
          __Pyx_INCREF(Py_None);
          __Pyx_GIVEREF(Py_None);
          PyTuple_SET_ITEM(__pyx_t_3, 4, Py_None);
          __pyx_t_6 = 0;
          __pyx_t_7 = 0;

          /* "bzrlib/_readdir_pyx.pyx":362
 *                     # statvalue (to keep)
 *                     # abspath (None, to set)
 *                     PyList_Append(result, (entry.d_ino, entry.d_name, None,             # <<<<<<<<<<<<<<
 *                         statvalue, None))
 *         finally:
 */
          __pyx_t_5 = PyList_Append(__pyx_v_result, __pyx_t_3); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "bzrlib/_readdir_pyx.pyx":340
 *                         raise_os_error(errno, "readdir: ", path)
 *                 name = entry.d_name
 *                 if not (name[0] == c"." and (             # <<<<<<<<<<<<<<
 *                     (name[1] == 0) or
 *                     (name[1] == c"." and name[2] == 0))
 */
        }
        __pyx_L15_continue:;
      }
    }

    /* "bzrlib/_readdir_pyx.pyx":365
 *                         statvalue, None))
 *         finally:
 *             if -1 == closedir(the_dir):             # <<<<<<<<<<<<<<
 *                 raise_os_error(errno, "closedir: ", path)
 *     finally:
 */
    /*finally:*/ {
      /*normal exit:*/{
        __pyx_t_2 = ((-1L == closedir(__pyx_v_the_dir)) != 0);
        if (__pyx_t_2) {

          /* "bzrlib/_readdir_pyx.pyx":366
 *         finally:
 *             if -1 == closedir(the_dir):
 *                 raise_os_error(errno, "closedir: ", path)             # <<<<<<<<<<<<<<
 *     finally:
 *         if -1 != orig_dir_fd:
 */
          __pyx_t_3 = __pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(errno, __pyx_k_closedir, __pyx_v_path); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L9_error;}
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "bzrlib/_readdir_pyx.pyx":365
 *                         statvalue, None))
 *         finally:
 *             if -1 == closedir(the_dir):             # <<<<<<<<<<<<<<
 *                 raise_os_error(errno, "closedir: ", path)
 *     finally:
 */
        }
        goto __pyx_L14;
      }
      /*exception exit:*/{
        __pyx_L13_error:;
        __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
        if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12) < 0)) __Pyx_ErrFetch(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_12);
        __Pyx_XGOTREF(__pyx_t_13);
        __Pyx_XGOTREF(__pyx_t_14);
        __Pyx_XGOTREF(__pyx_t_15);
        __pyx_t_5 = __pyx_lineno; __pyx_t_8 = __pyx_clineno; __pyx_t_9 = __pyx_filename;
        {
          __pyx_t_2 = ((-1L == closedir(__pyx_v_the_dir)) != 0);
          if (__pyx_t_2) {

            /* "bzrlib/_readdir_pyx.pyx":366
 *         finally:
 *             if -1 == closedir(the_dir):
 *                 raise_os_error(errno, "closedir: ", path)             # <<<<<<<<<<<<<<
 *     finally:
 *         if -1 != orig_dir_fd:
 */
            __pyx_t_3 = __pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(errno, __pyx_k_closedir, __pyx_v_path); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L37_error;}
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "bzrlib/_readdir_pyx.pyx":365
 *                         statvalue, None))
 *         finally:
 *             if -1 == closedir(the_dir):             # <<<<<<<<<<<<<<
 *                 raise_os_error(errno, "closedir: ", path)
 *     finally:
 */
          }
        }
        if (PY_MAJOR_VERSION >= 3) {
          __Pyx_XGIVEREF(__pyx_t_13);
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_ExceptionReset(__pyx_t_13, __pyx_t_14, __pyx_t_15);
        }
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_ErrRestore(__pyx_t_10, __pyx_t_11, __pyx_t_12);
        __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0;
        __pyx_lineno = __pyx_t_5; __pyx_clineno = __pyx_t_8; __pyx_filename = __pyx_t_9;
        goto __pyx_L9_error;
        __pyx_L37_error:;
        if (PY_MAJOR_VERSION >= 3) {
          __Pyx_XGIVEREF(__pyx_t_13);
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_ExceptionReset(__pyx_t_13, __pyx_t_14, __pyx_t_15);
        }
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0;
        goto __pyx_L9_error;
      }
      __pyx_L14:;
    }
  }

  /* "bzrlib/_readdir_pyx.pyx":368
 *                 raise_os_error(errno, "closedir: ", path)
 *     finally:
 *         if -1 != orig_dir_fd:             # <<<<<<<<<<<<<<
 *             failed = False
 *             if -1 == fchdir(orig_dir_fd):
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_2 = ((-1L != __pyx_v_orig_dir_fd) != 0);
      if (__pyx_t_2) {

        /* "bzrlib/_readdir_pyx.pyx":369
 *     finally:
 *         if -1 != orig_dir_fd:
 *             failed = False             # <<<<<<<<<<<<<<
 *             if -1 == fchdir(orig_dir_fd):
 *                 # try to close the original directory anyhow
 */
        __pyx_v_failed = 0;

        /* "bzrlib/_readdir_pyx.pyx":370
 *         if -1 != orig_dir_fd:
 *             failed = False
 *             if -1 == fchdir(orig_dir_fd):             # <<<<<<<<<<<<<<
 *                 # try to close the original directory anyhow
 *                 failed = True
 */
        __pyx_t_2 = ((-1L == fchdir(__pyx_v_orig_dir_fd)) != 0);
        if (__pyx_t_2) {

          /* "bzrlib/_readdir_pyx.pyx":372
 *             if -1 == fchdir(orig_dir_fd):
 *                 # try to close the original directory anyhow
 *                 failed = True             # <<<<<<<<<<<<<<
 *             if -1 == close(orig_dir_fd) or failed:
 *                 raise_os_error(errno, "return to orig_dir: ", "")
 */
          __pyx_v_failed = 1;

          /* "bzrlib/_readdir_pyx.pyx":370
 *         if -1 != orig_dir_fd:
 *             failed = False
 *             if -1 == fchdir(orig_dir_fd):             # <<<<<<<<<<<<<<
 *                 # try to close the original directory anyhow
 *                 failed = True
 */
        }

        /* "bzrlib/_readdir_pyx.pyx":373
 *                 # try to close the original directory anyhow
 *                 failed = True
 *             if -1 == close(orig_dir_fd) or failed:             # <<<<<<<<<<<<<<
 *                 raise_os_error(errno, "return to orig_dir: ", "")
 * 
 */
        __pyx_t_1 = ((-1L == close(__pyx_v_orig_dir_fd)) != 0);
        if (!__pyx_t_1) {
        } else {
          __pyx_t_2 = __pyx_t_1;
          goto __pyx_L42_bool_binop_done;
        }
        __pyx_t_1 = (__pyx_v_failed != 0);
        __pyx_t_2 = __pyx_t_1;
        __pyx_L42_bool_binop_done:;
        if (__pyx_t_2) {

          /* "bzrlib/_readdir_pyx.pyx":374
 *                 failed = True
 *             if -1 == close(orig_dir_fd) or failed:
 *                 raise_os_error(errno, "return to orig_dir: ", "")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
          __pyx_t_3 = __pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(errno, __pyx_k_return_to_orig_dir, __pyx_kp_s_); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 374; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

          /* "bzrlib/_readdir_pyx.pyx":373
 *                 # try to close the original directory anyhow
 *                 failed = True
 *             if -1 == close(orig_dir_fd) or failed:             # <<<<<<<<<<<<<<
 *                 raise_os_error(errno, "return to orig_dir: ", "")
 * 
 */
        }

        /* "bzrlib/_readdir_pyx.pyx":368
 *                 raise_os_error(errno, "closedir: ", path)
 *     finally:
 *         if -1 != orig_dir_fd:             # <<<<<<<<<<<<<<
 *             failed = False
 *             if -1 == fchdir(orig_dir_fd):
 */
      }
      goto __pyx_L10;
    }
    /*exception exit:*/{
      __pyx_L9_error:;
      __pyx_t_15 = 0; __pyx_t_14 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_12, &__pyx_t_11, &__pyx_t_10);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_15, &__pyx_t_14, &__pyx_t_13) < 0)) __Pyx_ErrFetch(&__pyx_t_15, &__pyx_t_14, &__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_10);
      __pyx_t_8 = __pyx_lineno; __pyx_t_5 = __pyx_clineno; __pyx_t_16 = __pyx_filename;
      {
        __pyx_t_2 = ((-1L != __pyx_v_orig_dir_fd) != 0);
        if (__pyx_t_2) {

          /* "bzrlib/_readdir_pyx.pyx":369
 *     finally:
 *         if -1 != orig_dir_fd:
 *             failed = False             # <<<<<<<<<<<<<<
 *             if -1 == fchdir(orig_dir_fd):
 *                 # try to close the original directory anyhow
 */
          __pyx_v_failed = 0;

          /* "bzrlib/_readdir_pyx.pyx":370
 *         if -1 != orig_dir_fd:
 *             failed = False
 *             if -1 == fchdir(orig_dir_fd):             # <<<<<<<<<<<<<<
 *                 # try to close the original directory anyhow
 *                 failed = True
 */
          __pyx_t_2 = ((-1L == fchdir(__pyx_v_orig_dir_fd)) != 0);
          if (__pyx_t_2) {

            /* "bzrlib/_readdir_pyx.pyx":372
 *             if -1 == fchdir(orig_dir_fd):
 *                 # try to close the original directory anyhow
 *                 failed = True             # <<<<<<<<<<<<<<
 *             if -1 == close(orig_dir_fd) or failed:
 *                 raise_os_error(errno, "return to orig_dir: ", "")
 */
            __pyx_v_failed = 1;

            /* "bzrlib/_readdir_pyx.pyx":370
 *         if -1 != orig_dir_fd:
 *             failed = False
 *             if -1 == fchdir(orig_dir_fd):             # <<<<<<<<<<<<<<
 *                 # try to close the original directory anyhow
 *                 failed = True
 */
          }

          /* "bzrlib/_readdir_pyx.pyx":373
 *                 # try to close the original directory anyhow
 *                 failed = True
 *             if -1 == close(orig_dir_fd) or failed:             # <<<<<<<<<<<<<<
 *                 raise_os_error(errno, "return to orig_dir: ", "")
 * 
 */
          __pyx_t_1 = ((-1L == close(__pyx_v_orig_dir_fd)) != 0);
          if (!__pyx_t_1) {
          } else {
            __pyx_t_2 = __pyx_t_1;
            goto __pyx_L49_bool_binop_done;
          }
          __pyx_t_1 = (__pyx_v_failed != 0);
          __pyx_t_2 = __pyx_t_1;
          __pyx_L49_bool_binop_done:;
          if (__pyx_t_2) {

            /* "bzrlib/_readdir_pyx.pyx":374
 *                 failed = True
 *             if -1 == close(orig_dir_fd) or failed:
 *                 raise_os_error(errno, "return to orig_dir: ", "")             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
            __pyx_t_3 = __pyx_f_6bzrlib_12_readdir_pyx_raise_os_error(errno, __pyx_k_return_to_orig_dir, __pyx_kp_s_); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 374; __pyx_clineno = __LINE__; goto __pyx_L45_error;}
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "bzrlib/_readdir_pyx.pyx":373
 *                 # try to close the original directory anyhow
 *                 failed = True
 *             if -1 == close(orig_dir_fd) or failed:             # <<<<<<<<<<<<<<
 *                 raise_os_error(errno, "return to orig_dir: ", "")
 * 
 */
          }

          /* "bzrlib/_readdir_pyx.pyx":368
 *                 raise_os_error(errno, "closedir: ", path)
 *     finally:
 *         if -1 != orig_dir_fd:             # <<<<<<<<<<<<<<
 *             failed = False
 *             if -1 == fchdir(orig_dir_fd):
 */
        }
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_11, __pyx_t_10);
      }
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_ErrRestore(__pyx_t_15, __pyx_t_14, __pyx_t_13);
      __pyx_t_15 = 0; __pyx_t_14 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_10 = 0;
      __pyx_lineno = __pyx_t_8; __pyx_clineno = __pyx_t_5; __pyx_filename = __pyx_t_16;
      goto __pyx_L1_error;
      __pyx_L45_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_11, __pyx_t_10);
      }
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_12 = 0; __pyx_t_11 = 0; __pyx_t_10 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L10:;
  }

  /* "bzrlib/_readdir_pyx.pyx":376
 *                 raise_os_error(errno, "return to orig_dir: ", "")
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "bzrlib/_readdir_pyx.pyx":274
 * 
 * 
 * cdef _read_dir(path):             # <<<<<<<<<<<<<<
 *     """Like os.listdir, this reads the contents of a directory.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("bzrlib._readdir_pyx._read_dir", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_statvalue);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_6bzrlib_12_readdir_pyx__Stat(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_6bzrlib_12_readdir_pyx__Stat(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_dev(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_6st_dev_1__get__(o);
}

static PyObject *__pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_ino(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_6st_ino_1__get__(o);
}

static PyObject *__pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_mode(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_7st_mode_1__get__(o);
}

static PyObject *__pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_ctime(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_8st_ctime_1__get__(o);
}

static PyObject *__pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_mtime(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_8st_mtime_1__get__(o);
}

static PyObject *__pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_7st_size_1__get__(o);
}

static PyMethodDef __pyx_methods_6bzrlib_12_readdir_pyx__Stat[] = {
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6bzrlib_12_readdir_pyx__Stat[] = {
  {(char *)"st_dev", __pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_dev, 0, 0, 0},
  {(char *)"st_ino", __pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_ino, 0, 0, 0},
  {(char *)"st_mode", __pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_mode, 0, 0, 0},
  {(char *)"st_ctime", __pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_ctime, 0, 0, 0},
  {(char *)"st_mtime", __pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_mtime, 0, 0, 0},
  {(char *)"st_size", __pyx_getprop_6bzrlib_12_readdir_pyx_5_Stat_st_size, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6bzrlib_12_readdir_pyx__Stat = {
  PyVarObject_HEAD_INIT(0, 0)
  "bzrlib._readdir_pyx._Stat", /*tp_name*/
  sizeof(struct __pyx_obj_6bzrlib_12_readdir_pyx__Stat), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6bzrlib_12_readdir_pyx__Stat, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_6bzrlib_12_readdir_pyx_5_Stat_1__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "Represent a 'stat' result.", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6bzrlib_12_readdir_pyx__Stat, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6bzrlib_12_readdir_pyx__Stat, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6bzrlib_12_readdir_pyx__Stat, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_6bzrlib_12_readdir_pyx_UTF8DirReader __pyx_vtable_6bzrlib_12_readdir_pyx_UTF8DirReader;

static PyObject *__pyx_tp_new_6bzrlib_12_readdir_pyx_UTF8DirReader(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *)o);
  p->__pyx_vtab = __pyx_vtabptr_6bzrlib_12_readdir_pyx_UTF8DirReader;
  return o;
}

static void __pyx_tp_dealloc_6bzrlib_12_readdir_pyx_UTF8DirReader(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_6bzrlib_12_readdir_pyx_UTF8DirReader[] = {
  {"kind_from_mode", (PyCFunction)__pyx_pw_6bzrlib_12_readdir_pyx_13UTF8DirReader_1kind_from_mode, METH_O, __pyx_doc_6bzrlib_12_readdir_pyx_13UTF8DirReader_kind_from_mode},
  {"top_prefix_to_starting_dir", (PyCFunction)__pyx_pw_6bzrlib_12_readdir_pyx_13UTF8DirReader_3top_prefix_to_starting_dir, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6bzrlib_12_readdir_pyx_13UTF8DirReader_2top_prefix_to_starting_dir},
  {"read_dir", (PyCFunction)__pyx_pw_6bzrlib_12_readdir_pyx_13UTF8DirReader_5read_dir, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6bzrlib_12_readdir_pyx_13UTF8DirReader_4read_dir},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_6bzrlib_12_readdir_pyx_UTF8DirReader = {
  PyVarObject_HEAD_INIT(0, 0)
  "bzrlib._readdir_pyx.UTF8DirReader", /*tp_name*/
  sizeof(struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6bzrlib_12_readdir_pyx_UTF8DirReader, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "A dir reader for utf8 file systems.", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6bzrlib_12_readdir_pyx_UTF8DirReader, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6bzrlib_12_readdir_pyx_UTF8DirReader, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "_readdir_pyx",
    __pyx_k_Wrapper_for_readdir_which_return, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_n_s_Exception, __pyx_k_Exception, sizeof(__pyx_k_Exception), 0, 0, 1, 1},
  {&__pyx_n_s_OSError, __pyx_k_OSError, sizeof(__pyx_k_OSError), 0, 0, 1, 1},
  {&__pyx_kp_s__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 1, 0},
  {&__pyx_kp_s__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 0, 1, 0},
  {&__pyx_n_s_block, __pyx_k_block, sizeof(__pyx_k_block), 0, 0, 1, 1},
  {&__pyx_n_s_bzrlib, __pyx_k_bzrlib, sizeof(__pyx_k_bzrlib), 0, 0, 1, 1},
  {&__pyx_n_s_chardev, __pyx_k_chardev, sizeof(__pyx_k_chardev), 0, 0, 1, 1},
  {&__pyx_n_s_directory, __pyx_k_directory, sizeof(__pyx_k_directory), 0, 0, 1, 1},
  {&__pyx_kp_s_failed_to_strcat, __pyx_k_failed_to_strcat, sizeof(__pyx_k_failed_to_strcat), 0, 0, 1, 0},
  {&__pyx_n_s_fifo, __pyx_k_fifo, sizeof(__pyx_k_fifo), 0, 0, 1, 1},
  {&__pyx_n_s_file, __pyx_k_file, sizeof(__pyx_k_file), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_os, __pyx_k_os, sizeof(__pyx_k_os), 0, 0, 1, 1},
  {&__pyx_n_s_osutils, __pyx_k_osutils, sizeof(__pyx_k_osutils), 0, 0, 1, 1},
  {&__pyx_n_s_prefix, __pyx_k_prefix, sizeof(__pyx_k_prefix), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_safe_utf8, __pyx_k_safe_utf8, sizeof(__pyx_k_safe_utf8), 0, 0, 1, 1},
  {&__pyx_n_s_socket, __pyx_k_socket, sizeof(__pyx_k_socket), 0, 0, 1, 1},
  {&__pyx_n_s_st_ctime, __pyx_k_st_ctime, sizeof(__pyx_k_st_ctime), 0, 0, 1, 1},
  {&__pyx_n_s_st_mode, __pyx_k_st_mode, sizeof(__pyx_k_st_mode), 0, 0, 1, 1},
  {&__pyx_n_s_st_mtime, __pyx_k_st_mtime, sizeof(__pyx_k_st_mtime), 0, 0, 1, 1},
  {&__pyx_n_s_st_size, __pyx_k_st_size, sizeof(__pyx_k_st_size), 0, 0, 1, 1},
  {&__pyx_n_s_symlink, __pyx_k_symlink, sizeof(__pyx_k_symlink), 0, 0, 1, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_top, __pyx_k_top, sizeof(__pyx_k_top), 0, 0, 1, 1},
  {&__pyx_n_s_unknown, __pyx_k_unknown, sizeof(__pyx_k_unknown), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_Exception = __Pyx_GetBuiltinName(__pyx_n_s_Exception); if (!__pyx_builtin_Exception) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_OSError = __Pyx_GetBuiltinName(__pyx_n_s_OSError); if (!__pyx_builtin_OSError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "bzrlib/_readdir_pyx.pyx":247
 *                 # PyString_Concat will have setup an exception, but how to get
 *                 # at it?
 *                 raise Exception("failed to strcat")             # <<<<<<<<<<<<<<
 *             PyTuple_SetItem_obj(atuple, 0, new_val_obj)
 *             # 1st None -> kind
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_failed_to_strcat); if (unlikely(!__pyx_tuple__3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "bzrlib/_readdir_pyx.pyx":263
 *                 # PyString_Concat will have setup an exception, but how to get
 *                 # at it?
 *                 raise Exception("failed to strcat")             # <<<<<<<<<<<<<<
 *             PyTuple_SetItem_obj(atuple, 4, new_val_obj)
 *         return result
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_failed_to_strcat); if (unlikely(!__pyx_tuple__4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 263; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_readdir_pyx(void); /*proto*/
PyMODINIT_FUNC init_readdir_pyx(void)
#else
PyMODINIT_FUNC PyInit__readdir_pyx(void); /*proto*/
PyMODINIT_FUNC PyInit__readdir_pyx(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__readdir_pyx(void)", 0);
  if (__Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_readdir_pyx", __pyx_methods, __pyx_k_Wrapper_for_readdir_which_return, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_bzrlib___readdir_pyx) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "bzrlib._readdir_pyx")) {
      if (unlikely(PyDict_SetItemString(modules, "bzrlib._readdir_pyx", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  __pyx_v_6bzrlib_12_readdir_pyx__directory = Py_None; Py_INCREF(Py_None);
  __pyx_v_6bzrlib_12_readdir_pyx__chardev = Py_None; Py_INCREF(Py_None);
  __pyx_v_6bzrlib_12_readdir_pyx__block = Py_None; Py_INCREF(Py_None);
  __pyx_v_6bzrlib_12_readdir_pyx__file = Py_None; Py_INCREF(Py_None);
  __pyx_v_6bzrlib_12_readdir_pyx__fifo = Py_None; Py_INCREF(Py_None);
  __pyx_v_6bzrlib_12_readdir_pyx__symlink = Py_None; Py_INCREF(Py_None);
  __pyx_v_6bzrlib_12_readdir_pyx__socket = Py_None; Py_INCREF(Py_None);
  __pyx_v_6bzrlib_12_readdir_pyx__unknown = Py_None; Py_INCREF(Py_None);
  __pyx_v_6bzrlib_12_readdir_pyx__safe_utf8 = Py_None; Py_INCREF(Py_None);
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_6bzrlib_12_readdir_pyx__Stat) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_6bzrlib_12_readdir_pyx__Stat.tp_print = 0;
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_6bzrlib_12_readdir_pyx__Stat, "__repr__"); if (unlikely(!wrapper)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_6bzrlib_12_readdir_pyx_5_Stat___repr__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_6bzrlib_12_readdir_pyx_5_Stat___repr__.doc = __pyx_doc_6bzrlib_12_readdir_pyx_5_Stat___repr__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_6bzrlib_12_readdir_pyx_5_Stat___repr__;
    }
  }
  #endif
  if (PyObject_SetAttrString(__pyx_m, "_Stat", (PyObject *)&__pyx_type_6bzrlib_12_readdir_pyx__Stat) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_6bzrlib_12_readdir_pyx__Stat = &__pyx_type_6bzrlib_12_readdir_pyx__Stat;
  __pyx_vtabptr_6bzrlib_12_readdir_pyx_UTF8DirReader = &__pyx_vtable_6bzrlib_12_readdir_pyx_UTF8DirReader;
  __pyx_vtable_6bzrlib_12_readdir_pyx_UTF8DirReader._kind_from_mode = (PyObject *(*)(struct __pyx_obj_6bzrlib_12_readdir_pyx_UTF8DirReader *, int))__pyx_f_6bzrlib_12_readdir_pyx_13UTF8DirReader__kind_from_mode;
  if (PyType_Ready(&__pyx_type_6bzrlib_12_readdir_pyx_UTF8DirReader) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type_6bzrlib_12_readdir_pyx_UTF8DirReader.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_6bzrlib_12_readdir_pyx_UTF8DirReader.tp_dict, __pyx_vtabptr_6bzrlib_12_readdir_pyx_UTF8DirReader) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (PyObject_SetAttrString(__pyx_m, "UTF8DirReader", (PyObject *)&__pyx_type_6bzrlib_12_readdir_pyx_UTF8DirReader) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 174; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_6bzrlib_12_readdir_pyx_UTF8DirReader = &__pyx_type_6bzrlib_12_readdir_pyx_UTF8DirReader;
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif

  /* "bzrlib/_readdir_pyx.pyx":20
 * 
 * 
 * import os             # <<<<<<<<<<<<<<
 * import sys
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_os, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_os, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "bzrlib/_readdir_pyx.pyx":21
 * 
 * import os
 * import sys             # <<<<<<<<<<<<<<
 * 
 * #python2.4 support
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_sys, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sys, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 21; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "bzrlib/_readdir_pyx.pyx":110
 * 
 * cdef object _directory
 * _directory = 'directory'             # <<<<<<<<<<<<<<
 * cdef object _chardev
 * _chardev = 'chardev'
 */
  __Pyx_INCREF(__pyx_n_s_directory);
  __Pyx_XGOTREF(__pyx_v_6bzrlib_12_readdir_pyx__directory);
  __Pyx_DECREF_SET(__pyx_v_6bzrlib_12_readdir_pyx__directory, __pyx_n_s_directory);
  __Pyx_GIVEREF(__pyx_n_s_directory);

  /* "bzrlib/_readdir_pyx.pyx":112
 * _directory = 'directory'
 * cdef object _chardev
 * _chardev = 'chardev'             # <<<<<<<<<<<<<<
 * cdef object _block
 * _block = 'block'
 */
  __Pyx_INCREF(__pyx_n_s_chardev);
  __Pyx_XGOTREF(__pyx_v_6bzrlib_12_readdir_pyx__chardev);
  __Pyx_DECREF_SET(__pyx_v_6bzrlib_12_readdir_pyx__chardev, __pyx_n_s_chardev);
  __Pyx_GIVEREF(__pyx_n_s_chardev);

  /* "bzrlib/_readdir_pyx.pyx":114
 * _chardev = 'chardev'
 * cdef object _block
 * _block = 'block'             # <<<<<<<<<<<<<<
 * cdef object _file
 * _file = 'file'
 */
  __Pyx_INCREF(__pyx_n_s_block);
  __Pyx_XGOTREF(__pyx_v_6bzrlib_12_readdir_pyx__block);
  __Pyx_DECREF_SET(__pyx_v_6bzrlib_12_readdir_pyx__block, __pyx_n_s_block);
  __Pyx_GIVEREF(__pyx_n_s_block);

  /* "bzrlib/_readdir_pyx.pyx":116
 * _block = 'block'
 * cdef object _file
 * _file = 'file'             # <<<<<<<<<<<<<<
 * cdef object _fifo
 * _fifo = 'fifo'
 */
  __Pyx_INCREF(__pyx_n_s_file);
  __Pyx_XGOTREF(__pyx_v_6bzrlib_12_readdir_pyx__file);
  __Pyx_DECREF_SET(__pyx_v_6bzrlib_12_readdir_pyx__file, __pyx_n_s_file);
  __Pyx_GIVEREF(__pyx_n_s_file);

  /* "bzrlib/_readdir_pyx.pyx":118
 * _file = 'file'
 * cdef object _fifo
 * _fifo = 'fifo'             # <<<<<<<<<<<<<<
 * cdef object _symlink
 * _symlink = 'symlink'
 */
  __Pyx_INCREF(__pyx_n_s_fifo);
  __Pyx_XGOTREF(__pyx_v_6bzrlib_12_readdir_pyx__fifo);
  __Pyx_DECREF_SET(__pyx_v_6bzrlib_12_readdir_pyx__fifo, __pyx_n_s_fifo);
  __Pyx_GIVEREF(__pyx_n_s_fifo);

  /* "bzrlib/_readdir_pyx.pyx":120
 * _fifo = 'fifo'
 * cdef object _symlink
 * _symlink = 'symlink'             # <<<<<<<<<<<<<<
 * cdef object _socket
 * _socket = 'socket'
 */
  __Pyx_INCREF(__pyx_n_s_symlink);
  __Pyx_XGOTREF(__pyx_v_6bzrlib_12_readdir_pyx__symlink);
  __Pyx_DECREF_SET(__pyx_v_6bzrlib_12_readdir_pyx__symlink, __pyx_n_s_symlink);
  __Pyx_GIVEREF(__pyx_n_s_symlink);

  /* "bzrlib/_readdir_pyx.pyx":122
 * _symlink = 'symlink'
 * cdef object _socket
 * _socket = 'socket'             # <<<<<<<<<<<<<<
 * cdef object _unknown
 * _unknown = 'unknown'
 */
  __Pyx_INCREF(__pyx_n_s_socket);
  __Pyx_XGOTREF(__pyx_v_6bzrlib_12_readdir_pyx__socket);
  __Pyx_DECREF_SET(__pyx_v_6bzrlib_12_readdir_pyx__socket, __pyx_n_s_socket);
  __Pyx_GIVEREF(__pyx_n_s_socket);

  /* "bzrlib/_readdir_pyx.pyx":124
 * _socket = 'socket'
 * cdef object _unknown
 * _unknown = 'unknown'             # <<<<<<<<<<<<<<
 * 
 * # add a typedef struct dirent dirent to workaround pyrex
 */
  __Pyx_INCREF(__pyx_n_s_unknown);
  __Pyx_XGOTREF(__pyx_v_6bzrlib_12_readdir_pyx__unknown);
  __Pyx_DECREF_SET(__pyx_v_6bzrlib_12_readdir_pyx__unknown, __pyx_n_s_unknown);
  __Pyx_GIVEREF(__pyx_n_s_unknown);

  /* "bzrlib/_readdir_pyx.pyx":169
 * 
 * 
 * from bzrlib import osutils             # <<<<<<<<<<<<<<
 * 
 * cdef object _safe_utf8
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_osutils);
  __Pyx_GIVEREF(__pyx_n_s_osutils);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_osutils);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_bzrlib, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_osutils); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_osutils, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "bzrlib/_readdir_pyx.pyx":172
 * 
 * cdef object _safe_utf8
 * _safe_utf8 = osutils.safe_utf8             # <<<<<<<<<<<<<<
 * 
 * cdef class UTF8DirReader:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_osutils); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_safe_utf8); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_6bzrlib_12_readdir_pyx__safe_utf8);
  __Pyx_DECREF_SET(__pyx_v_6bzrlib_12_readdir_pyx__safe_utf8, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "bzrlib/_readdir_pyx.pyx":1
 * # Copyright (C) 2006, 2008, 2009, 2010 Canonical Ltd             # <<<<<<<<<<<<<<
 * #
 * # This program is free software; you can redistribute it and/or modify
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init bzrlib._readdir_pyx", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init bzrlib._readdir_pyx");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject* args = PyTuple_Pack(1, arg);
    return (likely(args)) ? __Pyx_PyObject_Call(func, args, NULL) : NULL;
}
#endif

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_COMPILING_IN_CPYTHON
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
#else
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
#endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
#else
    PyErr_GetExcInfo(type, value, tb);
#endif
}
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(type, value, tb);
#endif
}

static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
#endif

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_ino_t(ino_t value) {
    const ino_t neg_one = (ino_t) -1, const_zero = (ino_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(ino_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(ino_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(ino_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(ino_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(ino_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(ino_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_off_t(off_t value) {
    const off_t neg_one = (off_t) -1, const_zero = (off_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(off_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(off_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(off_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(off_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(off_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(off_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
